@@ -1,6 +1,6 @@
 ##---------------------------------------------------------------------------##
 ##  File:
-##	$Id: qprint.pl,v 2.5 2001/09/17 16:09:32 ehood Exp $
+##	$Id: qprint.pl,v 2.6 2003/05/19 09:13:22 ehood Exp $
 ##  Authors:
 ##      Earl Hood       mhonarc@mhonarc.org
 ##	Alan Barrett	barrett@daisy.ee.und.ac.za
@@ -37,7 +37,7 @@ sub qprdecode {
 
     s/[^\S\r\n]*(\r?\n)/$1/g;	# remove trailing whitespace on each line
     s/\=\r?\n//g;		# remove soft linebreaks
-    s/=([0-9A-F]{2})/pack("H2",$1)/ge;	# convert hex codes
+    s/=([0-9A-Fa-f]{2})/pack('H2',$1)/ge;	# convert hex codes
     $_;				# return result
 }
 
